Package: Rmpi
Version: 0.7-3.3
Date: 2025-01-13
Title: Interface (Wrapper) to MPI (Message-Passing Interface)
Depends: R (>= 2.15.1)
Imports: parallel
Description: An interface (wrapper) to MPI. It also 
	     provides interactive R manager and worker environment.
License: GPL (>= 2)
URL: https://fisher.stats.uwo.ca/faculty/yu/Rmpi/
Maintainer: Hao Yu <hyu@stats.uwo.ca>
Packaged: 2025-01-13 15:26:37 UTC; hyuadm
Author: Hao Yu [aut, cre]
Authors@R: person(given = "Hao",
                    family = "Yu",
                    role = c("aut", "cre"),
                    email = "hyu@stats.uwo.ca")
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2025-01-13 16:50:02 UTC
