Rmpi is an R package providing an interface to MPI (Message-Passing Interface) 
API calls with interactive R slave functionalities. 

Copyright (C) 2014 Hao Yu <hyu@stats.uwo.ca>

******************************************************************************
Rmpi is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.
    
You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 59 Temple 
Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************

Rmpi requires R version 2.14.0 or higher, and a MPI distribution on unix 
platform, Microsoft MPI on windows platform, and OpenMPI on Mac OS. 
More information can be found at 
http://fisher.stats.uwo.ca/faculty/yu/Rmpi

